//***********************************************************
// iPod Information System
//
// Features:
//     - Playback Controls
//     - Displays Current Song Information
//     - Charge iPod (Not implemented in software)
//
// Version 0.3
// This should allow for play/pause, forward, backward,
// volume up and down, and enable control. When remote 
// is enabled, it should print Title, Artist, Album and 
// current song Time (being polled) to an Adafruit Monochron
// GLCD Screen.
//
// TO-DO:
//   [DONE]1. Try starting polling on play/pause button push and have
//      polling handler get title artist and album. May help with 6
//   [DONE]{DNE}2. Implement Volume Adjustment Control
//   [DONE]3. Print a play/pause image in upper left corner of display
//   [DONE]4. Print time somewhere
//   5. Implement library for helper functions
//   6. Investigate Problem of no displaying title at first play - apparently the clear are function clears all areas so it is only called in the title handler
//   [DONE]7. Implement function prototypes
//   [DONE]8. Change Font inclusions to only those used
//   [DONE]9. Set Artist and Album area to only display on line of text,
//     use same implementation as title area
//   [DONE]{Not Necessary}10. Make a button array and define BUTTON_PLAY, etc to index array
//   11. Scroll title text when its width is longer than the display[LATER]
//
// Based on the library for Apple Accessory Protocol
// writtten by David Findlay.
// Copyright (c) 2012
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//    - Redistributions of source code must retain the above copyright notice,
//      this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above copyright
//      notice, this list of conditions and the following disclaimer in the
//      documentation and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
// LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
// SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
// CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.
//*************************************************************
#include <AdvancedRemote.h>
#include <Wire.h>
#include <Bounce.h>
#include <string.h>
#include <glcd.h>
#include "fonts/SystemFont5x7.h"
#include "fonts/Arial14.h"

#define DEBOUNCE_TIME 50
#define BUTTON_POWER 12
#define BUTTON_PLAY A5 //Can be pin 13, but it is hard to use because the LED and resistor
#define BUTTON_FOR 2
#define BUTTON_BACK 3

///////////////////////////////////
// Global Variable Declarations //
/////////////////////////////////
unsigned long chosenSongIndexIntoPlaylist;
int isPlaying = 0;

AdvancedRemote advancedRemote;
gText titleArea = gText();
gText artistArea = gText();
gText albumArea = gText();
gText timeArea = gText();

Bounce buttonPower(BUTTON_POWER, DEBOUNCE_TIME); //Enables/Disable remote
Bounce buttonForward(BUTTON_FOR, DEBOUNCE_TIME);
Bounce buttonBack(BUTTON_BACK, DEBOUNCE_TIME);
Bounce buttonPlay(BUTTON_PLAY, DEBOUNCE_TIME); //Button 5, debounce 50ms

//////////////////////////
// Function Prototypes //
////////////////////////
/* Remote Callbacks */
void feedbackHandler(AdvancedRemote::Feedback feedback, byte cmd);
void titleHandler(const char *name);
void artistHandler(const char *name);
void albumHandler(const char *name);
void pollingHandler(AdvancedRemote::PollingCommand command, unsigned long playlistPositionOrElaspedTimeMs);
void currentPlaylistSongCountHandler(unsigned long count); //may not be needed
void playlistPositionHandler(unsigned long playlistPosition); //may not be needed
/* Helper Functions */
void printTime(const unsigned long ms);
void setupGLCD();
void drawPlayPause();

//////////////////////////////
// Arduino Set Up Function //
////////////////////////////
void setup()
{
	//Set up buttons
	pinMode(BUTTON_POWER, INPUT);
	pinMode(BUTTON_PLAY, INPUT);
	pinMode(BUTTON_FOR, INPUT);
	pinMode(BUTTON_BACK, INPUT);
	
	digitalWrite(BUTTON_POWER, HIGH);
	digitalWrite(BUTTON_PLAY, HIGH);
	digitalWrite(BUTTON_FOR, HIGH);
	digitalWrite(BUTTON_BACK, HIGH);
	
	//Initialize GLCD
	GLCD.Init();
	GLCD.SelectFont(Arial14);
	GLCD.ClearScreen();
	
	//Begin serial communication at 9600 bps
	Serial.begin(9600);
	
	//Setup Advanced Remote
	advancedRemote.setSerial(Serial);
	advancedRemote.setFeedbackHandler(feedbackHandler);
	advancedRemote.setTitleHandler(titleHandler);
	advancedRemote.setArtistHandler(artistHandler);
	advancedRemote.setAlbumHandler(albumHandler);
	advancedRemote.setPollingHandler(pollingHandler);
	advancedRemote.setCurrentPlaylistSongCountHandler(currentPlaylistSongCountHandler);
	advancedRemote.setPlaylistPositionHandler(playlistPositionHandler);
	advancedRemote.setup();
	advancedRemote.disable();
}

////////////////////////
// Main Program Loop //
//////////////////////
void loop()
{	
	advancedRemote.loop();
	
	//Toggle Advanced Remote On/Off
	if(buttonPower.update() && (buttonPower.read() == LOW))
	{
		if(advancedRemote.isCurrentlyEnabled())
		{
			advancedRemote.disable();
			GLCD.ClearScreen();
			GLCD.DrawString("Goodbye", GLCD.CenterX - GLCD.StringWidth("Goodbye")/2, GLCD.CenterY - 7);
			delay(3000);
			GLCD.ClearScreen();
		}
		else
		{
			advancedRemote.enable();
			GLCD.DrawString("Hello   Caitlin!", GLCD.CenterX - GLCD.StringWidth("Hello   Caitlin!")/2, GLCD.CenterY - 7);
			delay(3000);
			GLCD.ClearScreen();
			setupGLCD();
		}
	}
	
	//Toggle Play/Pause
	if(buttonPlay.update() && (buttonPlay.read() == LOW))
	{
		advancedRemote.getPlaylistPosition();
		drawPlayPause();		
		advancedRemote.controlPlayback(AdvancedRemote::PLAYBACK_CONTROL_PLAY_PAUSE);
		advancedRemote.getTitle(chosenSongIndexIntoPlaylist);
	}
	
	//Skip Forward to next track
	if(buttonForward.update() && (buttonForward.read() == LOW))
	{
		advancedRemote.controlPlayback(AdvancedRemote::PLAYBACK_CONTROL_SKIP_FORWARD);
	}
	
	//Skip Back to previous track
	if(buttonBack.update() && (buttonBack.read() == LOW))
	{
		advancedRemote.controlPlayback(AdvancedRemote::PLAYBACK_CONTROL_SKIP_BACKWARD);
	}
}

///////////////////////////
// Function Definitions //
/////////////////////////
/* Callback Function Definitions */
void feedbackHandler(AdvancedRemote::Feedback feedback, byte cmd)
{
	if(feedback != AdvancedRemote::FEEDBACK_SUCCESS)
		return;

	switch(cmd)
	{
		case AdvancedRemote::CMD_PLAYBACK_CONTROL:
		//advancedRemote.getTitle(chosenSongIndexIntoPlaylist);
		break;
	}
}
			
void titleHandler(const char *name)
{
	int numChars;
	char *temp;
	byte width = titleArea.StringWidth(name);
	
	titleArea.ClearArea();
	
	if(width < GLCD.Width)
	{
		titleArea.DefineArea((GLCD.CenterX - (width/2)), 10, (GLCD.CenterX + (width/2)), 25);
		titleArea.print(name);
	}
	else
	{
		numChars = GLCD.Width/titleArea.CharWidth(name[0]);
		temp = (char*) malloc(sizeof(char) * (numChars + 1));
		//May need to put blank characters in all temp
		titleArea.DefineArea(0, 10, 127, 25);
		strncpy(temp, name, numChars);
		temp[numChars] = '\0';
		titleArea.print(temp);
		free(temp);
	}
	
	advancedRemote.getArtist(chosenSongIndexIntoPlaylist);
}

void artistHandler(const char *name)
{
	int numChars;
	char *temp;
	byte width = artistArea.StringWidth(name);
	
	artistArea.ClearArea();
	
	if(width < GLCD.Width)
	{
		artistArea.DefineArea((GLCD.CenterX - (width/2)), 26, (GLCD.CenterX + (width/2)), 41); //26,41
		artistArea.print(name);
	}
	else
	{
		numChars = GLCD.Width/artistArea.CharWidth(name[0]);
		temp = (char*) malloc(sizeof(char) * (numChars + 1));
		//May need to put blank characters in all temp
		artistArea.DefineArea(0, 26, 127, 41);
		strncpy(temp, name, numChars);
		temp[numChars] = '\0';
		artistArea.print(temp);
		free(temp);
	}
	
	advancedRemote.getAlbum(chosenSongIndexIntoPlaylist);
}

void albumHandler(const char *name)
{
	int numChars;
	char *temp;
	byte width = albumArea.StringWidth(name);
	
	albumArea.ClearArea();
	
	if(width < GLCD.Width)
	{
		albumArea.DefineArea((GLCD.CenterX - (width/2)), 42, (GLCD.CenterX + (width/2)), 50);
		albumArea.print(name);
	}
	else
	{
		numChars = GLCD.Width/albumArea.CharWidth(name[0]);
		temp = (char*) malloc(sizeof(char) * (numChars + 1));
		//May need to put blank characters in all temp
		albumArea.DefineArea(0, 42, 127, 50);
		strncpy(temp, name, numChars);
		temp[numChars] = '\0';
		albumArea.print(temp);
		free(temp);
	}

	advancedRemote.setPollingMode(AdvancedRemote::POLLING_START);
}

void pollingHandler(AdvancedRemote::PollingCommand command, unsigned long playlistPositionOrElaspedTimeMs)
{  
	switch(command)
	{
		case AdvancedRemote::POLLING_TRACK_CHANGE:
		chosenSongIndexIntoPlaylist = playlistPositionOrElaspedTimeMs; //was ++
		advancedRemote.getTitle(chosenSongIndexIntoPlaylist);
		printTime(playlistPositionOrElaspedTimeMs);
		break;

		case AdvancedRemote::POLLING_ELAPSED_TIME:
		printTime(playlistPositionOrElaspedTimeMs);
		break;

		default:
		printTime(playlistPositionOrElaspedTimeMs);
	}
}

void currentPlaylistSongCountHandler(unsigned long count)
{
	advancedRemote.jumpToSongInCurrentPlaylist(chosenSongIndexIntoPlaylist);
}

void playlistPositionHandler(unsigned long playlistPosition)
{
	chosenSongIndexIntoPlaylist = playlistPosition;
}

/* Helper Function Definitions */
void printTime(const unsigned long ms)
{
	const unsigned long totalSecs = ms / 1000;
	const unsigned int mins = totalSecs / 60;
	const unsigned int partialSecs = totalSecs % 60;

	timeArea.ClearArea();
	timeArea.print(mins);
	timeArea.print(":");
	timeArea.print(partialSecs);
}

void setupGLCD()
{
	//Set up text areas
	titleArea.SelectFont(System5x7);
	artistArea.SelectFont(System5x7);
	albumArea.SelectFont(System5x7);
	timeArea.SelectFont(System5x7);
	timeArea.DefineArea(0, GLCD.Height - 8, 4, 1, System5x7);
}

void drawPlayPause()
{
	if(isPlaying)
	{
		isPlaying = 0;
		GLCD.FillRect(5, 0, 5, 6, WHITE);
		GLCD.FillRect(5, 0, 1, 5);
		GLCD.FillRect(9, 0, 1, 5);
	}
	else
	{
		isPlaying = 1;
		GLCD.FillRect(5, 0, 5, 6, WHITE);
		GLCD.FillRect(5, 0, 1, 5);
		GLCD.FillRect(7, 1, 1, 3);
		GLCD.FillRect(9, 2, 1, 1);
	}
}